package de.interpreter.brainfuck;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.widget.EditText;

/**
 * 
 * @author Daniel lschlegel
 * 2012/03
 * @license 2-BSDL
 *
 */

public class BfInterpreter {
	private byte[] memory = new byte[256 * 256];
	private char[] code = new char[256 * 256 + 1];
	private int memoryPointer;
	private int codeSize;
	private Context window;
	
	private StringBuilder output;
	
	public BfInterpreter(String code, Context window) {
		
		this.window = window;
		for (codeSize = 0; codeSize < code.length(); ++codeSize)
			this.code[codeSize] = code.charAt(codeSize);
		this.code[codeSize] = '\0';
		reset();
	}
	
	private void reset() {
		memoryPointer = 0;
		for (int i = 0; i < memory.length; ++i)
				memory[i] = '\0';
		output = new StringBuilder();
	}
	
	
	//TODO: checks for corrupt codes
	//TODO: performance
	//TODO: compatibility
	
	public String run(int ip) {
		while (true) {
			switch (code[ip]) {
			case '+':
				if (memory[memoryPointer] < 255)
					++memory[memoryPointer];
				break;
			case '-':
				if (memory[memoryPointer] > 0)
					--memory[memoryPointer];
				break;
			case '>':
				if (memoryPointer < 256 * 256 - 1)
					++memoryPointer;
				break;
			case '<':
				if (memoryPointer > 0)
					--memoryPointer;
				break;
			case '.':
				output.append((char) memory[memoryPointer]);
				break;
			case ',':
				dialog();
				break;
			case '[': 
				if (memory[memoryPointer] == 0) {
					++ip;
					int balance = 1;
					while (balance != 0) {
						if (code[ip] == '[')
							++balance;
						else if (code[ip] == ']')
							--balance;
						++ip;
					}
				}
				else 
					run(ip + 1);
				continue;
			case ']':
				return "";
			case '\0':
				// Exit
				return output.toString();
			default:
				break;
			}
			++ip;
		}
	}
	
	//TODO: bug, memory[memoryPointer] assign not working
	public void dialog() {
		AlertDialog.Builder alert = new AlertDialog.Builder(window);
		alert.setTitle("Input");
		alert.setMessage("Please enter a character:");

		// Set an EditText view to get user input 
		final EditText input = new EditText(window);
		alert.setView(input);

		alert.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
		public void onClick(DialogInterface dialog, int whichButton) {
			memory[memoryPointer] = (byte) input.getText().toString().charAt(0);
		}
		});

		alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
		  public void onClick(DialogInterface dialog, int whichButton) {
		    // Canceled.
		  }
		});

		alert.show();
	}
}
